/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/

/***********************************************************************
 * $Id: FXRbApp.cpp,v 1.7 2002/01/04 18:03:07 ljohnson Exp $
 ***********************************************************************/

#include "fx.h"
#include "fx3d.h"
#include "FXICOIcon.h"
#include "FXICOImage.h"
#include "FXJPGIcon.h"
#include "FXJPGImage.h"
#include "FXPNGIcon.h"
#include "FXPNGImage.h"
#include "FXTIFIcon.h"
#include "FXTIFImage.h"
#ifdef WITH_FXSCINTILLA
#include "Scintilla.h"
#include "FXScintilla.h"
#endif
#include "ruby.h"
#if defined(_MSC_VER) && defined(EXTERN)
#undef EXTERN
#define EXTERN
#endif
#include "rubysig.h" /* For CHECK_INTS */
#include "FXRuby.h"
#ifdef HAVE_SYS_TIME_H
#include <sys/time.h> /* For struct timeval */
#endif

// Message map
FXDEFMAP(FXRbApp) FXRbAppMap[]={
  FXMAPFUNC(SEL_CHORE,FXRbApp::ID_CHORE_THREADS,FXRbApp::onChoreThreads),
  };

// Class implementation
FXRbIMPLEMENT(FXRbApp,FXApp,FXRbAppMap,ARRAYNUMBER(FXRbAppMap))

// Constructor
FXRbApp::FXRbApp(const FXchar* appname,const FXchar* vendor) : FXApp(appname,vendor),threadsEnabled(TRUE),sleepTime(100),threadsChore(0){
  if(threadsEnabled){
    FXASSERT(threadsChore==0);
    threadsChore=addChore(this,ID_CHORE_THREADS);
    }
  }


// Enable threads
void FXRbApp::enableThreads(){
  if(!threadsEnabled){
    threadsEnabled=TRUE;
    threadsChore=addChore(this,ID_CHORE_THREADS);
    FXASSERT(threadsChore!=0);
    }
  }


// Disable threads
void FXRbApp::disableThreads(){
  if(threadsEnabled){
    threadsEnabled=FALSE;
    if(threadsChore!=0){
      removeChore(threadsChore);
      threadsChore=0;
      }
    }
  }


// Set sleep time
void FXRbApp::setSleepTime(FXuint ms){
  sleepTime=ms;
  }


// Get sleep time
FXuint FXRbApp::getSleepTime() const {
  return sleepTime;
  }


// Process threads
long FXRbApp::onChoreThreads(FXObject*,FXSelector,void*){
  // Pause for 'sleepTime' millseconds
  struct timeval wait;
  wait.tv_sec=0;
  wait.tv_usec=100*sleepTime;

  // Confirm that this thread can be interrupted, then go to sleep
  CHECK_INTS;
  if(!rb_thread_critical)
    rb_thread_wait_for(wait);

  // Re-register this chore for next time
  threadsChore=addChore(this,ID_CHORE_THREADS);
  FXASSERT(threadsChore!=0);

  // Back to work...
  return 1;
  }


// Destructor
FXRbApp::~FXRbApp(){
  FXRbDestroyAppSensitiveObjects();
  FXRbUnregisterRubyObj(this);
  }

