/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/

/***********************************************************************
 * $Id: FXRbApp.h,v 1.2 2002/01/02 22:23:55 ljohnson Exp $
 ***********************************************************************/

#ifndef FXRBAPP_H
#define FXRBAPP_H

#define IMPLEMENT_FXAPP_STUBS(cls,baseclass) \
  void cls::_create(){ \
    baseclass::create(); \
    } \
  void cls::create(){ \
    FXRbCallVoidMethod(this,"create"); \
    } \
  void cls::_detach(){ \
    baseclass::detach(); \
    } \
  void cls::detach(){ \
    FXRbCallVoidMethod(this,"detach"); \
    } \
  void cls::_destroy(){ \
    baseclass::destroy(); \
    } \
  void cls::destroy(){ \
    FXRbCallVoidMethod(this,"destroy"); \
    } \
  void cls::_init(VALUE arr,FXbool connect){ \
    int argc=RARRAY(arr)->len+1; \
    char** argv; \
    if(FXMALLOC(&argv,char*,argc)){ \
      int i; \
      argv[0]=NULL; \
      for(i=1; i<argc; i++){ \
        argv[i]=STR2CSTR(rb_ary_entry(arr,i-1)); \
        } \
      baseclass::init(argc,argv,connect); \
      while(RARRAY(arr)->len!=0){ \
        rb_ary_pop(arr); \
        } \
      for(i=1; i<argc; i++){ \
        rb_ary_push(arr,to_ruby(argv[i])); \
        } \
      FXFREE(&argv); \
      } \
    } \
  void cls::init(int& argc,char** argv,FXbool connect){ \
    int i; \
    VALUE arr=rb_ary_new(); \
    for(i=1; i<argc; i++){ \
      rb_ary_push(arr,to_ruby(argv[i])); \
      } \
    FXRbCallVoidMethod(this,"init",arr,connect); \
    argc=RARRAY(arr)->len+1; \
    for(i=1; i<argc; i++){ \
      argv[i]=STR2CSTR(rb_ary_entry(arr,i-1)); \
      } \
    } \
  void cls::_exit(FXint code){ \
    baseclass::exit(code); \
    } \
  void cls::exit(FXint code){ \
    FXRbCallVoidMethod(this,"exit",code); \
    }


class FXRbApp : public FXApp {
  FXDECLARE(FXRbApp)
protected:
  FXbool threadsEnabled;
  FXuint sleepTime;
  FXChore* threadsChore;
protected:
  FXRbApp(){}
#include "FXRbObjectVirtuals.h"
#include "FXRbAppVirtuals.h"
public:
  enum {
    ID_CHORE_THREADS=FXApp::ID_LAST,
    ID_LAST
    };
public:
  long onChoreThreads(FXObject*,FXSelector,void*);
public:
  // Constructor
  FXRbApp(const FXchar* name,const FXchar* vendor);

  // Enable support for multithreaded applications
  void enableThreads();

  // Disable support for multithreaded applications
  void disableThreads();

  // Set sleep time (in milliseconds)
  void setSleepTime(FXuint ms);

  // Get sleep time (in milliseconds)
  FXuint getSleepTime() const;

  // Mark dependencies for GC
  static void markfunc(FXApp *app);

  // Destructor
  virtual ~FXRbApp();
  };

#endif
