/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/

/***********************************************************************
 * $Id: FXRbCursor.h,v 1.1 2001/07/12 19:44:17 ljohnson Exp $
 ***********************************************************************/

#ifndef FXRBCURSOR_H
#define FXRBCURSOR_H

#define IMPLEMENT_FXCURSOR_STUBS(cls,baseclass) \
  void cls::_savePixels(FXStream& store) const { \
    baseclass::savePixels(store); \
    } \
  void cls::savePixels(FXStream& store) const { \
    FXRbCallVoidMethod(this,"savePixels",store); \
    } \
  void cls::_loadPixels(FXStream& store){ \
    baseclass::loadPixels(store); \
    } \
  void cls::loadPixels(FXStream& store){ \
    FXRbCallVoidMethod(this,"loadPixels",store); \
    }

class FXRbCursor : public FXCursor {
  FXDECLARE(FXRbCursor)
protected:
  FXRbCursor(){}
#include "FXRbObjectVirtuals.h"
#include "FXRbIdVirtuals.h"
#include "FXRbCursorVirtuals.h"
public:
  /// Make stock cursor
  FXRbCursor(FXApp* a,FXStockCursor curid=CURSOR_ARROW) : FXCursor(a,curid){
    FXRbRegisterAppSensitiveObject(this);
    }

  /// Make cursor from source and mask; cursor size should at most 32x32 for portability!
  FXRbCursor(FXApp* a,const void* src,const void* msk,FXint w=32,FXint h=32,FXint hx=-1,FXint hy=-1) : FXCursor(a,src,msk,w,h,hx,hy){
    FXRbRegisterAppSensitiveObject(this);
    }

  // Mark dependencies for the GC
  static void markfunc(FXCursor* cursor);

  // Destructor
  virtual ~FXRbCursor(){
    FXRbUnregisterRubyObj(this);
    FXRbUnregisterAppSensitiveObject(this);
    }
  };

#endif
