/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/

/***********************************************************************
 * $Id: FXRbGLObject.h,v 1.3 2001/08/16 18:21:55 ljohnson Exp $
 ***********************************************************************/

#ifndef FXRBGLOBJECT_H
#define FXRBGLOBJECT_H

#define IMPLEMENT_FXGLOBJECT_STUBS(cls,baseclass) \
  FXGLObject* cls::_copy(){ \
    return baseclass::copy(); \
    } \
  FXGLObject* cls::copy(){ \
    return FXRbCallGLObjectMethod(this,"copy"); \
    } \
  FXRange cls::_bounds(){ \
    FXRange box; \
    baseclass::bounds(box); \
    return box; \
    } \
  void cls::bounds(FXRange& box){ \
    box=FXRbCallRangeMethod(this,"bounds"); \
    } \
  void cls::_draw(FXGLViewer* viewer){ \
    baseclass::draw(viewer); \
    } \
  void cls::draw(FXGLViewer* viewer){ \
    FXRbCallVoidMethod(this,"draw",viewer); \
    } \
  void cls::_hit(FXGLViewer* viewer){ \
    baseclass::hit(viewer); \
    } \
  void cls::hit(FXGLViewer* viewer){ \
    FXRbCallVoidMethod(this,"hit",viewer); \
    } \
  FXGLObject* cls::_identify(FXuint* path){ \
    return baseclass::identify(path); \
    } \
  FXGLObject* cls::identify(FXuint* path){ \
    return baseclass::identify(path); \
    } \
  FXbool cls::_canDrag() const { \
    return baseclass::canDrag(); \
    } \
  FXbool cls::canDrag() const { \
    return FXRbCallBoolMethod(this,"canDrag"); \
    } \
  FXbool cls::_canDelete() const{ \
    return baseclass::canDelete(); \
    } \
  FXbool cls::canDelete() const{ \
    return FXRbCallBoolMethod(this,"canDelete"); \
    } \
  FXbool cls::_drag(FXGLViewer* viewer,FXint fx,FXint fy,FXint tx,FXint ty){ \
    return baseclass::drag(viewer,fx,fy,tx,ty); \
    } \
  FXbool cls::drag(FXGLViewer* viewer,FXint fx,FXint fy,FXint tx,FXint ty){ \
    return FXRbCallBoolMethod(this,"drag",viewer,fx,fy,tx,ty); \
    }


class FXRbGLObject : public FXGLObject {
  FXDECLARE(FXRbGLObject)
#include "FXRbObjectVirtuals.h"
#include "FXRbGLObjectVirtuals.h"
public:
  // Is this GL object owned by a GL group?
  FXbool owned;
public:
  // Constructor
  FXRbGLObject() : owned(FALSE){}

  // Mark dependencies for the GC
  static void markfunc(FXGLObject* self);

  // Destroy this object if it's not owned
  static void freefunc(FXRbGLObject* self);

  // Destructor
  virtual ~FXRbGLObject(){
    FXRbUnregisterRubyObj(this);
    }
  };


class FXRbGLGroup : public FXGLGroup {
  FXDECLARE(FXRbGLGroup)
#include "FXRbObjectVirtuals.h"
#include "FXRbGLObjectVirtuals.h"
public:
  // Constructor
  FXRbGLGroup(){}

  // Mark dependencies for the GC
  static void markfunc(FXGLGroup* self);

  // Destructor
  virtual ~FXRbGLGroup(){
    FXRbUnregisterRubyObj(this);
    }
  };


class FXRbGLPoint : public FXGLPoint {
  FXDECLARE(FXRbGLPoint)
#include "FXRbObjectVirtuals.h"
#include "FXRbGLObjectVirtuals.h"
public:
  // Default constructor
  FXRbGLPoint(){}

  // Construct with specified coordinates
  FXRbGLPoint(FXfloat x,FXfloat y,FXfloat z) : FXGLPoint(x,y,z){}

  // Mark dependencies for the GC
  static void markfunc(FXGLPoint* self);

  // Destructor
  virtual ~FXRbGLPoint(){
    FXRbUnregisterRubyObj(this);
    }
  };


class FXRbGLLine : public FXGLLine {
  FXDECLARE(FXRbGLLine)
#include "FXRbObjectVirtuals.h"
#include "FXRbGLObjectVirtuals.h"
public:
  // Default constructor
  FXRbGLLine(){}

  // Construct with specified endpoints
  FXRbGLLine(FXfloat fx,FXfloat fy,FXfloat fz,FXfloat tx,FXfloat ty,FXfloat tz) : FXGLLine(fx,fy,fz,tx,ty,tz){}

  // Mark dependencies for the GC
  static void markfunc(FXGLLine* self);

  // Destructor
  virtual ~FXRbGLLine(){
    FXRbUnregisterRubyObj(this);
    }
  };

#endif
