/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/

/***********************************************************************
 * $Id: FXRbHeader.h,v 1.2 2001/07/13 15:31:37 ljohnson Exp $
 ***********************************************************************/

#ifndef FXRBHEADER_H
#define FXRBHEADER_H

class FXRbHeader : public FXHeader {
  FXDECLARE(FXRbHeader)
protected:
  FXRbHeader(){}
#include "FXRbObjectVirtuals.h"
#include "FXRbIdVirtuals.h"
#include "FXRbDrawableVirtuals.h"
#include "FXRbWindowVirtuals.h"
public:
  /// Construct new header control
  FXRbHeader(FXComposite* p,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=HEADER_NORMAL,FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint pl=DEFAULT_PAD,FXint pr=DEFAULT_PAD,FXint pt=DEFAULT_PAD,FXint pb=DEFAULT_PAD) : FXHeader(p,tgt,sel,opts,x,y,w,h,pl,pr,pt,pb){}

  // Mark dependencies for the GC
  static void markfunc(FXHeader* self);

  // Destructor
  virtual ~FXRbHeader(){
    FXRbUnregisterRubyObj(this);
    }
  };


#define IMPLEMENT_FXHEADERITEM_STUBS(cls,baseclass) \
  void cls::_setText(const FXString& text){ \
    baseclass::setText(text); \
    } \
  void cls::setText(const FXString& text){ \
    FXRbCallVoidMethod(this,"setText",text); \
    } \
  void cls::_setIcon(FXIcon* icn){ \
    baseclass::setIcon(icn); \
    } \
  void cls::setIcon(FXIcon* icn){ \
    FXRbCallVoidMethod(this,"setIcon",icn); \
    } \
  FXint cls::_getWidth(const FXHeader* header) const { \
    return baseclass::getWidth(header); \
    } \
  FXint cls::getWidth(const FXHeader* header) const { \
    return FXRbCallIntMethod(this,"getWidth",header); \
    } \
  FXint cls::_getHeight(const FXHeader* header) const { \
    return baseclass::getHeight(header); \
    } \
  FXint cls::getHeight(const FXHeader* header) const { \
    return FXRbCallIntMethod(this,"getHeight",header); \
    } \
  void cls::_create(){ \
    baseclass::create(); \
    } \
  void cls::create(){ \
    FXRbCallVoidMethod(this,"create"); \
    } \
  void cls::_detach(){ \
    baseclass::detach(); \
    } \
  void cls::detach(){ \
    FXRbCallVoidMethod(this,"detach"); \
    } \
  void cls::_destroy(){ \
    baseclass::destroy(); \
    } \
  void cls::destroy(){ \
    FXRbCallVoidMethod(this,"destroy"); \
    }


class FXRbHeaderItem : public FXHeaderItem {
  FXDECLARE(FXRbHeaderItem)
protected:
  FXRbHeaderItem(){}
#include "FXRbObjectVirtuals.h"
#include "FXRbHeaderItemVirtuals.h"
public:
  // Is this header item owned by an FXHeader yet?
  FXbool owned;
public:
  /// Construct new item with given text, icon, size, and user-data
  FXRbHeaderItem(const FXString& text,FXIcon* ic=NULL,FXint s=0,void* ptr=NULL) : FXHeaderItem(text,ic,s,ptr),owned(FALSE){}

  // Mark dependencies for the GC
  static void markfunc(FXHeaderItem* self);

  // Destroy this object if it's not owned
  static void freefunc(FXRbHeaderItem* self);

  // Destructor
  virtual ~FXRbHeaderItem(){
    FXRbUnregisterRubyObj(this);
    }
  };

#endif
