/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/

/***********************************************************************
 * $Id: FXRbIconList.h,v 1.3 2001/07/13 22:11:41 ljohnson Exp $
 ***********************************************************************/

#ifndef FXRBICONLIST_H
#define FXRBICONLIST_H

class FXRbIconList : public FXIconList {
  FXDECLARE(FXRbIconList)
protected:
  FXRbIconList(){}
#include "FXRbObjectVirtuals.h"
#include "FXRbIdVirtuals.h"
#include "FXRbDrawableVirtuals.h"
#include "FXRbWindowVirtuals.h"
#include "FXRbScrollAreaVirtuals.h"
public:
  /// Construct icon list
  FXRbIconList(FXComposite *p,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=ICONLIST_NORMAL,FXint x=0,FXint y=0,FXint w=0,FXint h=0) : FXIconList(p,tgt,sel,opts,x,y,w,h){
    setSortFunc(FXRbIconList::sortProc);
    }

  // Mark dependencies for the GC
  static void markfunc(FXIconList* self);

  // Sort function stand-in
  static FXint sortProc(const FXIconItem* a,const FXIconItem* b);

  // Destructor
  virtual ~FXRbIconList(){
    FXRbUnregisterRubyObj(this);
    }
  };


#define IMPLEMENT_FXICONITEM_STUBS(cls,baseclass) \
  void cls::_setText(const FXString& txt){ \
    baseclass::setText(txt); \
    } \
  void cls::setText(const FXString& txt){ \
    FXRbCallVoidMethod(this,"setText",txt); \
    } \
  void cls::_setBigIcon(FXIcon* icn){ \
    baseclass::setBigIcon(icn); \
    } \
  void cls::setBigIcon(FXIcon* icn){ \
    FXRbCallVoidMethod(this,"setBigIcon",icn); \
    } \
  void cls::_setMiniIcon(FXIcon* icn){ \
    baseclass::setMiniIcon(icn); \
    } \
  void cls::setMiniIcon(FXIcon* icn){ \
    FXRbCallVoidMethod(this,"setMiniIcon",icn); \
    } \
  void cls::_setFocus(FXbool focus){ \
    baseclass::setFocus(focus); \
    } \
  void cls::setFocus(FXbool focus){ \
    FXRbCallVoidMethod(this,"setFocus",focus); \
    } \
  void cls::_setSelected(FXbool selected){ \
    baseclass::setSelected(selected); \
    } \
  void cls::setSelected(FXbool selected){ \
    FXRbCallVoidMethod(this,"setSelected",selected); \
    } \
  void cls::_setEnabled(FXbool enabled){ \
    baseclass::setEnabled(enabled); \
    } \
  void cls::setEnabled(FXbool enabled){ \
    FXRbCallVoidMethod(this,"setEnabled",enabled); \
    } \
  void cls::_setDraggable(FXbool draggable){ \
    baseclass::setDraggable(draggable); \
    } \
  void cls::setDraggable(FXbool draggable){ \
    FXRbCallVoidMethod(this,"setDraggable",draggable); \
    } \
  void cls::_setIconOwned(FXuint owned){ \
    baseclass::setIconOwned(owned); \
    } \
  void cls::setIconOwned(FXuint owned){ \
    FXRbCallVoidMethod(this,"setIconOwned",owned); \
    } \
  FXint cls::_getWidth(const FXIconList* list) const { \
    return baseclass::getWidth(list); \
    } \
  FXint cls::getWidth(const FXIconList* list) const { \
    return FXRbCallIntMethod(this,"getWidth",list); \
    } \
  FXint cls::_getHeight(const FXIconList* list) const { \
    return baseclass::getHeight(list); \
    } \
  FXint cls::getHeight(const FXIconList* list) const { \
    return FXRbCallIntMethod(this,"getHeight",list); \
    } \
  void cls::_create(){ \
    baseclass::create(); \
    } \
  void cls::create(){ \
    FXRbCallVoidMethod(this,"create"); \
    } \
  void cls::_detach(){ \
    baseclass::detach(); \
    } \
  void cls::detach(){ \
    FXRbCallVoidMethod(this,"detach"); \
    } \
  void cls::_destroy(){ \
    baseclass::destroy(); \
    } \
  void cls::destroy(){ \
    FXRbCallVoidMethod(this,"destroy"); \
    }


class FXRbIconItem : public FXIconItem {
  FXDECLARE(FXRbIconItem)
protected:
  FXRbIconItem(){}
#include "FXRbObjectVirtuals.h"
#include "FXRbIconItemVirtuals.h"
public:
  // Is this icon item owned by an FXIconList yet?
  FXbool owned;
public:
  // Constructor
  FXRbIconItem(const FXString& text,FXIcon* bi=NULL,FXIcon* mi=NULL,void* ptr=NULL) : FXIconItem(text,bi,mi,ptr),owned(FALSE){}

  // Mark dependencies for the GC
  static void markfunc(FXIconItem* self);

  // Destroy this object if it's not owned
  static void freefunc(FXRbIconItem* self);

  // Destructor
  virtual ~FXRbIconItem(){
    FXRbUnregisterRubyObj(this);
    }
  };

#endif
