/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/

/***********************************************************************
 * $Id: FXRbImage.h,v 1.1 2001/07/12 19:44:17 ljohnson Exp $
 ***********************************************************************/

#ifndef FXRBIMAGE_H
#define FXRBIMAGE_H

#define IMPLEMENT_FXIMAGE_STUBS(cls,baseclass) \
  void cls::_restore(){ \
    baseclass::restore(); \
    } \
  void cls::restore(){ \
    FXRbCallVoidMethod(this,"restore"); \
    } \
  void cls::_render(){ \
    baseclass::render(); \
    } \
  void cls::render(){ \
    FXRbCallVoidMethod(this,"render"); \
    } \
  void cls::_scale(FXint w,FXint h){ \
    baseclass::scale(w,h); \
    } \
  void cls::scale(FXint w,FXint h){ \
    FXRbCallVoidMethod(this,"scale",w,h); \
    } \
  void cls::_mirror(FXbool horizontal,FXbool vertical){ \
    baseclass::mirror(horizontal,vertical); \
    } \
  void cls::mirror(FXbool horizontal,FXbool vertical){ \
    FXRbCallVoidMethod(this,"mirror",horizontal,vertical); \
    } \
  void cls::_rotate(FXint degrees){ \
    baseclass::rotate(degrees); \
    } \
  void cls::rotate(FXint degrees){ \
    FXRbCallVoidMethod(this,"rotate",degrees); \
    } \
  void cls::_crop(FXint x,FXint y,FXint w,FXint h){ \
    baseclass::crop(x,y,w,h); \
    } \
  void cls::crop(FXint x,FXint y,FXint w,FXint h){ \
    FXRbCallVoidMethod(this,"crop",x,y,w,h); \
    } \
  void cls::_savePixels(FXStream& store) const { \
    baseclass::savePixels(store); \
    } \
  void cls::savePixels(FXStream& store) const { \
    FXRbCallVoidMethod(this,"savePixels",store); \
    } \
  void cls::_loadPixels(FXStream& store){ \
    baseclass::loadPixels(store); \
    } \
  void cls::loadPixels(FXStream& store){ \
    FXRbCallVoidMethod(this,"loadPixels",store); \
    }


class FXRbImage : public FXImage {
  FXDECLARE(FXRbImage)
protected:
  FXRbImage(){}
#include "FXRbObjectVirtuals.h"
#include "FXRbIdVirtuals.h"
#include "FXRbDrawableVirtuals.h"
#include "FXRbImageVirtuals.h"
public:
  /// Create an image
  FXRbImage(FXApp* a,const void *pix=NULL,FXuint opts=0,FXint w=1,FXint h=1):FXImage(a,pix,opts,w,h){
    FXRbRegisterAppSensitiveObject(this);
    }

  // Mark dependencies for the GC
  static void markfunc(FXImage* image);

  // Destructor
  virtual ~FXRbImage(){
    FXRbUnregisterRubyObj(this);
    FXRbUnregisterAppSensitiveObject(this);
    }
  };

#endif
