/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/

/***********************************************************************
 * $Id: FXRbList.h,v 1.3 2001/07/13 22:11:41 ljohnson Exp $
 ***********************************************************************/

#ifndef FXRBLIST_H
#define FXRBLIST_H

#define IMPLEMENT_FXLISTITEM_STUBS(cls,baseclass) \
  void cls::_create(){ \
    baseclass::create(); \
    } \
  void cls::create(){ \
    FXRbCallVoidMethod(this,"create"); \
    } \
  void cls::_detach(){ \
    baseclass::detach(); \
    } \
  void cls::detach(){ \
    FXRbCallVoidMethod(this,"detach"); \
    } \
  void cls::_destroy(){ \
    baseclass::destroy(); \
    } \
  void cls::destroy(){ \
    FXRbCallVoidMethod(this,"destroy"); \
    } \
  void cls::_setText(const FXString& txt){ \
    baseclass::setText(txt); \
    } \
  void cls::setText(const FXString& txt){ \
    FXRbCallVoidMethod(this,"setText",txt); \
    } \
  void cls::_setIcon(FXIcon* icn){ \
    baseclass::setIcon(icn); \
    } \
  void cls::setIcon(FXIcon* icn){ \
    FXRbCallVoidMethod(this,"setIcon",icn); \
    } \
  void cls::_setFocus(FXbool focus){ \
    baseclass::setFocus(focus); \
    } \
  void cls::setFocus(FXbool focus){ \
    FXRbCallVoidMethod(this,"setFocus",focus); \
    } \
  void cls::_setSelected(FXbool selected){ \
    baseclass::setSelected(selected); \
    } \
  void cls::setSelected(FXbool selected){ \
    FXRbCallVoidMethod(this,"setSelected",selected); \
    } \
  void cls::_setEnabled(FXbool enabled){ \
    baseclass::setEnabled(enabled); \
    } \
  void cls::setEnabled(FXbool enabled){ \
    FXRbCallVoidMethod(this,"setEnabled",enabled); \
    } \
  void cls::_setDraggable(FXbool draggable){ \
    baseclass::setDraggable(draggable); \
    } \
  void cls::setDraggable(FXbool draggable){ \
    FXRbCallVoidMethod(this,"setDraggable",draggable); \
    } \
  void cls::_setIconOwned(FXuint owned){ \
    baseclass::setIconOwned(owned); \
    } \
  void cls::setIconOwned(FXuint owned){ \
    FXRbCallVoidMethod(this,"setIconOwned",owned); \
    } \
  FXint cls::_getWidth(const FXList* list) const { \
    return baseclass::getWidth(list); \
    } \
  FXint cls::getWidth(const FXList* list) const { \
    return FXRbCallIntMethod(this,"getWidth",list); \
    } \
  FXint cls::_getHeight(const FXList* list) const { \
    return baseclass::getHeight(list); \
    } \
  FXint cls::getHeight(const FXList* list) const { \
    return FXRbCallIntMethod(this,"getHeight",list); \
    }


class FXRbListItem : public FXListItem {
  FXDECLARE(FXRbListItem)
protected:
  FXRbListItem(){}
#include "FXRbObjectVirtuals.h"
#include "FXRbListItemVirtuals.h"
public:
  // Is this list item owned by an FXList yet?
  FXbool owned;
public:
  // Constructor
  FXRbListItem(const FXString& text,FXIcon* ic=NULL,void* ptr=NULL) : FXListItem(text,ic,ptr),owned(FALSE){}

  // Mark dependencies for the GC
  static void markfunc(FXListItem* self);

  // Destroy this object if it's not owned
  static void freefunc(FXRbListItem* self);

  // Destructor
  virtual ~FXRbListItem(){
    FXRbUnregisterRubyObj(this);
    }
  };


class FXRbList : public FXList {
  FXDECLARE(FXRbList)
protected:
  FXRbList(){}
#include "FXRbObjectVirtuals.h"
#include "FXRbIdVirtuals.h"
#include "FXRbDrawableVirtuals.h"
#include "FXRbWindowVirtuals.h"
#include "FXRbScrollAreaVirtuals.h"
public:
  /// Construct a list with nvis visible items; the list is initially empty
  FXRbList(FXComposite *p,FXint nvis,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=LIST_NORMAL,FXint x=0,FXint y=0,FXint w=0,FXint h=0) : FXList(p,nvis,tgt,sel,opts,x,y,w,h){
    setSortFunc(FXRbList::sortProc);
    }

  // Sort function stand-in
  static FXint sortProc(const FXListItem* a,const FXListItem* b);

  // Mark dependencies for the GC
  static void markfunc(FXList* self);

  // Destructor
  virtual ~FXRbList(){
    FXRbUnregisterRubyObj(this);
    }
  };

#endif
