/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/

/***********************************************************************
 * $Id: FXRbTable.h,v 1.4 2001/10/24 16:05:44 ljohnson Exp $
 ***********************************************************************/

#ifndef FXRBTABLE_H
#define FXRBTABLE_H

#define IMPLEMENT_FXTABLEITEM_STUBS(cls,baseclass) \
  void cls::_setText(const FXString& txt){ \
    baseclass::setText(txt); \
    } \
  void cls::setText(const FXString& txt){ \
    FXRbCallVoidMethod(this,"setText",txt); \
    } \
  void cls::_setIcon(FXIcon* icn){ \
    baseclass::setIcon(icn); \
    } \
  void cls::setIcon(FXIcon* icn){ \
    FXRbCallVoidMethod(this,"setIcon",icn); \
    } \
  void cls::_setFocus(FXbool focus){ \
    baseclass::setFocus(focus); \
    } \
  void cls::setFocus(FXbool focus){ \
    FXRbCallVoidMethod(this,"setFocus",focus); \
    } \
  void cls::_setSelected(FXbool selected){ \
    baseclass::setSelected(selected); \
    } \
  void cls::setSelected(FXbool selected){ \
    FXRbCallVoidMethod(this,"setSelected",selected); \
    } \
  void cls::_setEnabled(FXbool enabled){ \
    baseclass::setEnabled(enabled); \
    } \
  void cls::setEnabled(FXbool enabled){ \
    FXRbCallVoidMethod(this,"setEnabled",enabled); \
    } \
  void cls::_setDraggable(FXbool draggable){ \
    baseclass::setDraggable(draggable); \
    } \
  void cls::setDraggable(FXbool draggable){ \
    FXRbCallVoidMethod(this,"setDraggable",draggable); \
    } \
  void cls::_setIconOwned(FXuint owned){ \
    baseclass::setIconOwned(owned); \
    } \
  void cls::setIconOwned(FXuint owned){ \
    FXRbCallVoidMethod(this,"setIconOwned",owned); \
    } \
  FXint cls::_getWidth(const FXTable* table) const { \
    return baseclass::getWidth(table); \
    } \
  FXint cls::getWidth(const FXTable* table) const { \
    return FXRbCallIntMethod(this,"getWidth",table); \
    } \
  FXint cls::_getHeight(const FXTable* table) const { \
    return baseclass::getHeight(table); \
    } \
  FXint cls::getHeight(const FXTable* table) const { \
    return FXRbCallIntMethod(this,"getHeight",table); \
    } \
  void cls::_create(){ \
    baseclass::create(); \
    } \
  void cls::create(){ \
    FXRbCallVoidMethod(this,"create"); \
    } \
  void cls::_detach(){ \
    baseclass::detach(); \
    } \
  void cls::detach(){ \
    FXRbCallVoidMethod(this,"detach"); \
    } \
  void cls::_destroy(){ \
    baseclass::destroy(); \
    } \
  void cls::destroy(){ \
    FXRbCallVoidMethod(this,"destroy"); \
    }


class FXRbTableItem : public FXTableItem {
  FXDECLARE(FXRbTableItem)
protected:
  FXRbTableItem(){}
#include "FXRbObjectVirtuals.h"
#include "FXRbTableItemVirtuals.h"
public:
  // Is this table item owned by an FXTable yet?
  FXbool owned;
public:
  // Constructor
  FXRbTableItem(const FXString& text,FXIcon* ic=NULL,void* ptr=NULL) : FXTableItem(text,ic,ptr),owned(FALSE){}

  // Mark dependencies for the GC
  static void markfunc(FXTableItem* self);

  // Destroy this object if it's not owned
  static void freefunc(FXRbTableItem* self);

  // Destructor
  virtual ~FXRbTableItem(){
    FXRbUnregisterRubyObj(this);
    }
  };


#define IMPLEMENT_FXTABLE_STUBS(cls,baseclass) \
  void cls::_setLeadingRows(FXint leadrows){ \
    baseclass::setLeadingRows(leadrows); \
    } \
  void cls::setLeadingRows(FXint leadrows){ \
    FXRbCallVoidMethod(this,"setLeadingRows",leadrows); \
    } \
  void cls::_setLeadingCols(FXint leadcols){ \
    baseclass::setLeadingCols(leadcols); \
    } \
  void cls::setLeadingCols(FXint leadcols){ \
    FXRbCallVoidMethod(this,"setLeadingCols",leadcols); \
    } \
  void cls::_setTrailingRows(FXint trailrows){ \
    baseclass::setTrailingRows(trailrows); \
    } \
  void cls::setTrailingRows(FXint trailrows){ \
    FXRbCallVoidMethod(this,"setTrailingRows",trailrows); \
    } \
  void cls::_setTrailingCols(FXint trailcols){ \
    baseclass::setTrailingCols(trailcols); \
    } \
  void cls::setTrailingCols(FXint trailcols){ \
    FXRbCallVoidMethod(this,"setTrailingCols",trailcols); \
    } \
  void cls::_insertRows(FXint row,FXint nr,FXbool notify){ \
    baseclass::insertRows(row,nr,notify); \
    } \
  void cls::insertRows(FXint row,FXint nr,FXbool notify){ \
    FXRbCallVoidMethod(this,"insertRows",row,nr,notify); \
    } \
  void cls::_insertColumns(FXint col,FXint nc,FXbool notify){ \
    baseclass::insertColumns(col,nc,notify); \
    } \
  void cls::insertColumns(FXint col,FXint nc,FXbool notify){ \
    FXRbCallVoidMethod(this,"insertColumns",col,nc,notify); \
    } \
  void cls::_removeRows(FXint row,FXint nr,FXbool notify){ \
    baseclass::removeRows(row,nr,notify); \
    } \
  void cls::removeRows(FXint row,FXint nr,FXbool notify){ \
    FXRbCallVoidMethod(this,"removeRows",row,nr,notify); \
    } \
  void cls::_removeColumns(FXint col,FXint nc,FXbool notify){ \
    baseclass::removeColumns(col,nc,notify); \
    } \
  void cls::removeColumns(FXint col,FXint nc,FXbool notify){ \
    FXRbCallVoidMethod(this,"removeColumns",col,nc,notify); \
    } \
  void cls::_removeItem(FXint row,FXint col,FXbool notify){ \
    baseclass::removeItem(row,col,notify); \
    } \
  void cls::removeItem(FXint row,FXint col,FXbool notify){ \
    FXRbCallVoidMethod(this,"removeItem",row,col,notify); \
    }


class FXRbTable : public FXTable {
  FXDECLARE(FXRbTable)
protected:
  FXRbTable(){}
#include "FXRbObjectVirtuals.h"
#include "FXRbIdVirtuals.h"
#include "FXRbDrawableVirtuals.h"
#include "FXRbWindowVirtuals.h"
#include "FXRbScrollAreaVirtuals.h"
#include "FXRbTableVirtuals.h"
public:
  /// Make new table with nr visible rows and nc visible columns; the table
  /// is initially empty, i.e. contains no cells (nrows=0, ncols=0)
  FXRbTable(FXComposite *p,FXint nr,FXint nc,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint pl=DEFAULT_MARGIN,FXint pr=DEFAULT_MARGIN,FXint pt=DEFAULT_MARGIN,FXint pb=DEFAULT_MARGIN) : FXTable(p,nr,nc,tgt,sel,opts,x,y,w,h,pl,pr,pt,pb){}

  // Mark dependencies for the GC
  static void markfunc(FXTable* self);

  // Destructor
  virtual ~FXRbTable(){
    FXRbUnregisterRubyObj(this);
    }
  };

#endif
