/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/

/***********************************************************************
 * $Id: FXRbTreeList.h,v 1.3 2001/07/13 22:11:42 ljohnson Exp $
 ***********************************************************************/

#ifndef FXRBTREELIST_H
#define FXRBTREELIST_H

#define IMPLEMENT_FXTREEITEM_STUBS(cls,baseclass) \
  void cls::_setText(const FXString& txt){ \
    baseclass::setText(txt); \
    } \
  void cls::setText(const FXString& txt){ \
    FXRbCallVoidMethod(this,"setText",txt); \
    } \
  void cls::_setOpenIcon(FXIcon* icn){ \
    baseclass::setOpenIcon(icn); \
    } \
  void cls::setOpenIcon(FXIcon* icn){ \
    FXRbCallVoidMethod(this,"setOpenIcon",icn); \
    } \
  void cls::_setClosedIcon(FXIcon* icn){ \
    baseclass::setClosedIcon(icn); \
    } \
  void cls::setClosedIcon(FXIcon* icn){ \
    FXRbCallVoidMethod(this,"setClosedIcon",icn); \
    } \
  void cls::_setFocus(FXbool focus){ \
    baseclass::setFocus(focus); \
    } \
  void cls::setFocus(FXbool focus){ \
    FXRbCallVoidMethod(this,"setFocus",focus); \
    } \
  void cls::_setSelected(FXbool selected){ \
    baseclass::setSelected(selected); \
    } \
  void cls::setSelected(FXbool selected){ \
    FXRbCallVoidMethod(this,"setSelected",selected); \
    } \
  void cls::_setOpened(FXbool opened){ \
    baseclass::setOpened(opened); \
    } \
  void cls::setOpened(FXbool opened){ \
    FXRbCallVoidMethod(this,"setOpened",opened); \
    } \
  void cls::_setExpanded(FXbool expanded){ \
    baseclass::setExpanded(expanded); \
    } \
  void cls::setExpanded(FXbool expanded){ \
    FXRbCallVoidMethod(this,"setExpanded",expanded); \
    } \
  void cls::_setEnabled(FXbool enabled){ \
    baseclass::setEnabled(enabled); \
    } \
  void cls::setEnabled(FXbool enabled){ \
    FXRbCallVoidMethod(this,"setEnabled",enabled); \
    } \
  void cls::_setDraggable(FXbool draggable){ \
    baseclass::setDraggable(draggable); \
    } \
  void cls::setDraggable(FXbool draggable){ \
    FXRbCallVoidMethod(this,"setDraggable",draggable); \
    } \
  void cls::_setIconOwned(FXuint owned){ \
    baseclass::setIconOwned(owned); \
    } \
  void cls::setIconOwned(FXuint owned){ \
    FXRbCallVoidMethod(this,"setIconOwned",owned); \
    } \
  FXint cls::_getWidth(const FXTreeList* list) const { \
    return baseclass::getWidth(list); \
    } \
  FXint cls::getWidth(const FXTreeList* list) const { \
    return FXRbCallIntMethod(this,"getWidth",list); \
    } \
  FXint cls::_getHeight(const FXTreeList* list) const { \
    return baseclass::getHeight(list); \
    } \
  FXint cls::getHeight(const FXTreeList* list) const { \
    return FXRbCallIntMethod(this,"getHeight",list); \
    } \
  void cls::_create(){ \
    baseclass::create(); \
    } \
  void cls::create(){ \
    FXRbCallVoidMethod(this,"create"); \
    } \
  void cls::_detach(){ \
    baseclass::detach(); \
    } \
  void cls::detach(){ \
    FXRbCallVoidMethod(this,"detach"); \
    } \
  void cls::_destroy(){ \
    baseclass::destroy(); \
    } \
  void cls::destroy(){ \
    FXRbCallVoidMethod(this,"destroy"); \
    }


class FXRbTreeItem : public FXTreeItem {
  FXDECLARE(FXRbTreeItem)
protected:
  FXRbTreeItem(){}
#include "FXRbObjectVirtuals.h"
#include "FXRbTreeItemVirtuals.h"
public:
  // Is this tree item owned by an FXTreeList yet?
  FXbool owned;
public:
  // Constructor
  FXRbTreeItem(const FXString& text,FXIcon* oi=NULL,FXIcon* ci=NULL,void* ptr=NULL) : FXTreeItem(text,oi,ci,ptr),owned(FALSE){}

  // Mark dependencies for the GC
  static void markfunc(FXTreeItem* self);

  // Destroy this object if it's not owned
  static void freefunc(FXRbTreeItem* self);

  // Destructor
  virtual ~FXRbTreeItem(){
    FXRbUnregisterRubyObj(this);
    }
  };


#define IMPLEMENT_FXTREELIST_STUBS(cls,baseclass) \
  FXTreeItem* cls::_getItemAt(FXint x,FXint y) const { \
    return baseclass::getItemAt(x,y); \
    } \
  FXTreeItem* cls::getItemAt(FXint x,FXint y) const { \
    return baseclass::getItemAt(x,y); \
    }


class FXRbTreeList : public FXTreeList {
  FXDECLARE(FXRbTreeList)
protected:
  FXRbTreeList(){}
#include "FXRbObjectVirtuals.h"
#include "FXRbIdVirtuals.h"
#include "FXRbDrawableVirtuals.h"
#include "FXRbWindowVirtuals.h"
#include "FXRbScrollAreaVirtuals.h"
#include "FXRbTreeListVirtuals.h"
public:
  /// Construct a tree list with nvis visible items; the tree list is initially empty
  FXRbTreeList(FXComposite *p,FXint nvis,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=TREELIST_NORMAL,FXint x=0,FXint y=0,FXint w=0,FXint h=0) : FXTreeList(p,nvis,tgt,sel,opts,x,y,w,h){
    setSortFunc(FXRbTreeList::sortProc);
    }

  // Sort function stand-in
  static FXint sortProc(const FXTreeItem* a,const FXTreeItem* b);

  // Mark dependencies for the GC
  static void markfunc(FXTreeList* self);

  // Destructor
  virtual ~FXRbTreeList(){
    FXRbUnregisterRubyObj(this);
    }
  };

#endif
