/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/

/***********************************************************************
 * $Id: FXRbWindow.h,v 1.3 2001/11/26 17:47:45 ljohnson Exp $
 ***********************************************************************/

#ifndef FXRBWINDOW_H
#define FXRBWINDOW_H

#define IMPLEMENT_FXWINDOW_STUBS(cls,baseclass) \
  FXint cls::_getDefaultWidth(){ \
    return baseclass::getDefaultWidth(); \
    } \
  FXint cls::getDefaultWidth(){ \
    return FXRbCallIntMethod(this,"getDefaultWidth"); \
    } \
  FXint cls::_getDefaultHeight(){ \
    return baseclass::getDefaultHeight(); \
    } \
  FXint cls::getDefaultHeight(){ \
    return FXRbCallIntMethod(this,"getDefaultHeight"); \
    } \
  FXint cls::_getWidthForHeight(FXint givenheight){ \
    return baseclass::getWidthForHeight(givenheight); \
    } \
  FXint cls::getWidthForHeight(FXint givenheight){ \
    return FXRbCallIntMethod(this,"getWidthForHeight",givenheight); \
    } \
  FXint cls::_getHeightForWidth(FXint givenwidth){ \
    return baseclass::getHeightForWidth(givenwidth); \
    } \
  FXint cls::getHeightForWidth(FXint givenwidth){ \
    return FXRbCallIntMethod(this,"getHeightForWidth",givenwidth); \
    } \
  FXbool cls::_canFocus() const { \
    return baseclass::canFocus(); \
    } \
  FXbool cls::canFocus() const { \
    return FXRbCallBoolMethod(this,"canFocus"); \
    } \
  void cls::_setFocus(){ \
    baseclass::setFocus(); \
    } \
  void cls::setFocus(){ \
    FXRbCallVoidMethod(this,"setFocus"); \
    } \
  void cls::_killFocus(){ \
    baseclass::killFocus(); \
    } \
  void cls::killFocus(){ \
    FXRbCallVoidMethod(this,"killFocus"); \
    } \
  void cls::_setDefault(FXbool enable){ \
    baseclass::setDefault(enable); \
    } \
  void cls::setDefault(FXbool enable){ \
    FXRbCallVoidMethod(this,"setDefault",enable); \
    } \
  void cls::_enable(){ \
    baseclass::enable(); \
    } \
  void cls::enable(){ \
    FXRbCallVoidMethod(this,"enable"); \
    } \
  void cls::_disable(){ \
    baseclass::disable(); \
    } \
  void cls::disable(){ \
    FXRbCallVoidMethod(this,"disable"); \
    } \
  void cls::_raise(){ \
    baseclass::raise(); \
    } \
  void cls::raise(){ \
    FXRbCallVoidMethod(this,"raise"); \
    } \
  void cls::_lower(){ \
    baseclass::lower(); \
    } \
  void cls::lower(){ \
    FXRbCallVoidMethod(this,"lower"); \
    } \
  void cls::_move(FXint x,FXint y){ \
    baseclass::move(x,y); \
    } \
  void cls::move(FXint x,FXint y){ \
    FXRbCallVoidMethod(this,"move",x,y); \
    } \
  void cls::_position(FXint x,FXint y,FXint w,FXint h){ \
    baseclass::position(x,y,w,h); \
    } \
  void cls::position(FXint x,FXint y,FXint w,FXint h){ \
    FXRbCallVoidMethod(this,"position",x,y,w,h); \
    } \
  void cls::_recalc(){ \
    baseclass::recalc(); \
    } \
  void cls::recalc(){ \
    FXRbCallVoidMethod(this,"recalc"); \
    } \
  void cls::_reparent(FXComposite* newparent){ \
    baseclass::reparent(newparent); \
    } \
  void cls::reparent(FXComposite* newparent){ \
    FXRbCallVoidMethod(this,"reparent",newparent); \
    } \
  void cls::_show(){ \
    baseclass::show(); \
    } \
  void cls::show(){ \
    FXRbCallVoidMethod(this,"show"); \
    } \
  void cls::_hide(){ \
    baseclass::hide(); \
    } \
  void cls::hide(){ \
    FXRbCallVoidMethod(this,"hide"); \
    } \
  FXbool cls::_isComposite() const { \
    return baseclass::isComposite(); \
    } \
  FXbool cls::isComposite() const { \
    return FXRbCallBoolMethod(this,"isComposite"); \
    } \
  FXbool cls::_contains(FXint parentx,FXint parenty) const{ \
    return baseclass::contains(parentx,parenty); \
    } \
  FXbool cls::contains(FXint parentx,FXint parenty) const{ \
    return FXRbCallBoolMethod(this,"contains",parentx,parenty); \
    } \
  FXbool cls::_doesSaveUnder() const { \
    return baseclass::doesSaveUnder(); \
    } \
  FXbool cls::doesSaveUnder() const { \
    return FXRbCallBoolMethod(this,"doesSaveUnder"); \
    } \
  void cls::_setBackColor(FXColor clr) { \
    baseclass::setBackColor(clr); \
    } \
  void cls::setBackColor(FXColor clr) { \
    FXRbCallVoidMethod(this,"setBackColor",clr); \
    } \
  void cls::_layout() { \
    baseclass::layout(); \
    } \
  void cls::layout() { \
    FXRbCallVoidMethod(this,"layout"); \
    }


class FXRbWindow : public FXWindow {
  FXDECLARE(FXRbWindow)
#include "FXRbObjectVirtuals.h"
#include "FXRbIdVirtuals.h"
#include "FXRbDrawableVirtuals.h"
#include "FXRbWindowVirtuals.h"
protected:
  FXRbWindow(){}
  FXRbWindow(FXApp* a,FXVisual *vis):FXWindow(a,vis){}
  FXRbWindow(FXApp* a,FXWindow* own,FXuint opts,FXint x,FXint y,FXint w,FXint h):FXWindow(a,own,opts,x,y,w,h){}
public:
  // Constructor
  FXRbWindow(FXComposite* p,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0) : FXWindow(p,opts,x,y,w,h){}

  // Mark dependencies for the GC
  static void markfunc(FXWindow* win);

  // Destructor
  virtual ~FXRbWindow(){
    FXRbUnregisterRubyObj(this);
    }
  };

#endif
