/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/

class FXVisual;

/**
* Drawable is an abstract base class for any surface that can be
* drawn upon, such as a FXWindow, or FXImage.
*/
class FXDrawable : public FXId {
protected:
  FXVisual     *visual;                 // Visual for this window
  FXint         width;                  // Width
  FXint         height;                 // Height
protected:
  FXDrawable();
  FXDrawable(FXApp* a,FXint w,FXint h);
public:

  /// Width of drawable
  FXint getWidth() const;

  /// Height of drawable
  FXint getHeight() const;

  /// Get the visual
  FXVisual* getVisual() const;

  /// Change visual
  void setVisual(FXVisual* vis);

  /// Resize drawable to the specified width and height 
  virtual void resize(FXint w,FXint h);

  /// Cleanup
  virtual ~FXDrawable();

public:
  %pragma(ruby) markfunc = "FXRbDrawable::markfunc";
  };


class FXRbDrawable : public FXDrawable {
protected:
  // Default constructor
  FXRbDrawable();

  // Construct with this width and height
  FXRbDrawable(FXApp* a,FXint w,FXint h);

public:
  // Destructor
  virtual ~FXRbDrawable();

public:
  DECLARE_FXOBJECT_VIRTUALS
  DECLARE_FXID_VIRTUALS
  DECLARE_FXDRAWABLE_VIRTUALS

public:
  %pragma(ruby) markfunc = "FXRbDrawable::markfunc";
  };

