/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/

/// GLCanvas, an area drawn by another object
class FXGLCanvas : public FXCanvas {
protected:
  void        *ctx;     // GL Context
protected:
  FXGLCanvas();
public:

  /**
  * Construct an OpenGL-capable canvas, with its own private display list.
  */
  FXGLCanvas(FXComposite* p,FXGLVisual *vis,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0);

  /**
  * Construct an OpenGL-capable canvas, sharing display
  * list with another GL canvas.  This canvas becomes a member
  * of a display list share group.  All members of the display
  * list share group have to have the same visual.
  */
  // FXGLCanvas(FXComposite* p,FXGLVisual *vis,FXGLCanvas* sharegroup,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0); OVERLOAD

  /// Return TRUE if it is sharing display lists
  FXbool isShared() const;
  
  /// Make OpenGL context current prior to performing OpenGL commands
  virtual FXbool makeCurrent();
  
  /// Make OpenGL context non current
  virtual FXbool makeNonCurrent();

  /// Return TRUE if context is current
  virtual FXbool isCurrent() const;

  /// Get GL context handle
  // void* getContext() const; // FIXME
  
  /// Swap front and back buffer
  virtual void swapBuffers();
  
  /// Destructor
  virtual ~FXGLCanvas();

public:
  %pragma(ruby) markfunc = "FXRbGLCanvas::markfunc";
  };


class FXRbGLCanvas : public FXGLCanvas {
public:
  /**
  * Construct an OpenGL-capable canvas, with its own private display list.
  */
  FXRbGLCanvas(FXComposite* p,FXGLVisual *vis,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0);
  %pragma(python) addtomethod = "__init__:FXPyRegister(self)"

  /**
  * Construct an OpenGL-capable canvas, sharing display
  * list with another GL canvas.  This canvas becomes a member
  * of a display list share group.  All members of the display
  * list share group have to have the same visual.
  */
  // FXRbGLCanvas(FXComposite* p,FXGLVisual *vis,FXGLCanvas* sharegroup,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0); OVERLOAD

  // Destructor
  virtual ~FXRbGLCanvas();

public:
  DECLARE_FXOBJECT_VIRTUALS
  DECLARE_FXID_VIRTUALS
  DECLARE_FXDRAWABLE_VIRTUALS
  DECLARE_FXWINDOW_VIRTUALS
  DECLARE_FXGLCANVAS_VIRTUALS

public:
  %pragma(ruby) markfunc = "FXRbGLCanvas::markfunc";
  };


#ifdef SWIGPYTHON

%inline %{
  FXPyGLCanvas* CreateGLCanvas1(FXComposite* p,FXGLVisual *vis,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0) {
    return new FXPyGLCanvas(p,vis,tgt,sel,opts,x,y,w,h);
  }
  
  FXPyGLCanvas* CreateGLCanvas2(FXComposite* p,FXGLVisual *vis,FXGLCanvas *sharegroup,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0) {
    return new FXPyGLCanvas(p,vis,sharegroup,tgt,sel,opts,x,y,w,h);
  }
%}

#endif
