/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/

class FXGLViewer;

class FXGLObject;


/// Basic OpenGL object
class FXGLObject : public FXObject {
public:
  enum {
    ID_LAST=10000       // Leaving ample room for FXGLViewer subclasses
    };
public:

  /// Constructor
  FXGLObject();

  /// Clone this object
  virtual FXGLObject* copy();

#ifdef SWIGRUBY
  /// Called by the viewer to get bounds for this object
  %addmethods {
    FXRange bounds() {
      FXRange box;
      self->bounds(box);
      return box;
    }
  }
#else
  %addmethods {
    // Called by the viewer to get bounds for this object
    FXRange* bounds(FXRange& box) {
      self->bounds(box);
      return &box;
    }
  }
#endif

  /// Draw this object in a viewer
  virtual void draw(FXGLViewer* viewer);

  /// Draw this object for hit-testing purposes
  virtual void hit(FXGLViewer* viewer);

  /// Identify sub-object given path
  virtual FXGLObject* identify(FXuint* path); // FIXME

  /// Return true if this object can be dragged around
  virtual FXbool canDrag() const;

  /// Return true if this object can be deleted from the scene
  virtual FXbool canDelete() const;

  /// Drag this object from one position to another
  virtual FXbool drag(FXGLViewer* viewer,FXint fx,FXint fy,FXint tx,FXint ty);

  /// Destructor
  // virtual ~FXGLObject();

public:
  %pragma(ruby) markfunc = "FXRbGLObject::markfunc";
  };


class FXRbGLObject : public FXGLObject {
public:
  // Constructor
  FXRbGLObject();
  %pragma(python) addtomethod = "__init__:FXPyRegister(self)"

  // Destructor
  %pragma(ruby) freefunc = "FXRbGLObject::freefunc";
  virtual ~FXRbGLObject();

public:
  DECLARE_FXOBJECT_VIRTUALS
  DECLARE_FXGLOBJECT_VIRTUALS

public:
  %pragma(ruby) markfunc = "FXRbGLObject::markfunc";
  };


/// Group object
class FXGLGroup : public FXGLObject {
public:

  /// Constructor
  FXGLGroup();

  /// Return list of children
  // FXGLObjectList& getList();

  /// Return number of children
  FXint no() const;

  /// Child at position
  FXGLObject* child(FXint pos) const;

  %addmethods {
    /// Insert child object at given position
    void insert(FXint pos,FXGLObject* obj){
      if(obj->isMemberOf(FXMETACLASS(FXRbGLObject)))
        ((FXRbGLObject*)obj)->owned=TRUE;
      self->insert(pos,obj);
      }

    /// Prepend child object
    void prepend(FXGLObject* obj){
      if(obj->isMemberOf(FXMETACLASS(FXRbGLObject)))
        ((FXRbGLObject*)obj)->owned=TRUE;
      self->prepend(obj);
      }

    /// Append child object
    void append(FXGLObject* obj){
      if(obj->isMemberOf(FXMETACLASS(FXRbGLObject)))
        ((FXRbGLObject*)obj)->owned=TRUE;
      self->append(obj);
      }

    /// Replace child object
    void replace(FXint pos,FXGLObject* obj){
      if(obj->isMemberOf(FXMETACLASS(FXRbGLObject)))
        ((FXRbGLObject*)obj)->owned=TRUE;
      self->replace(pos,obj);
      }
  }

  /// Remove child object
  %name(removeObj) void remove(FXGLObject* obj);

  /// Remove child object at given position
  %name(removePos) void remove(FXint pos);

  /// Remove all children
  void clear();

  /// Destructor
  virtual ~FXGLGroup();

public:
  %pragma(ruby) markfunc = "FXRbGLGroup::markfunc";
  };


class FXRbGLGroup : public FXGLGroup {
public:
  // Constructor
  FXRbGLGroup();
  %pragma(python) addtomethod = "__init__:FXPyRegister(self)"

  // Destructor
  virtual ~FXRbGLGroup();

public:
  DECLARE_FXOBJECT_VIRTUALS
  DECLARE_FXGLOBJECT_VIRTUALS

public:
  %pragma(ruby) markfunc = "FXRbGLGroup::markfunc";
  };
