/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/

/// Homogeneous matrix (single-precision version)
class FXHMat {
public:
  // Constructor
  FXHMat(FXfloat a00,FXfloat a01,FXfloat a02,FXfloat a03,
         FXfloat a10,FXfloat a11,FXfloat a12,FXfloat a13,
         FXfloat a20,FXfloat a21,FXfloat a22,FXfloat a23,
         FXfloat a30,FXfloat a31,FXfloat a32,FXfloat a33);

  // Operators
  %addmethods {
    // Add matrices
    FXHMat __add__(const FXHMat& other) {
      return (*self) + other;
      }

    // Subtract matrices
    FXHMat __sub__(const FXHMat& other) {
      return (*self) - other;
      }

    // Multiply matrices
    FXHMat __mul__(const FXHMat& other) {
      return (*self) * other;
      }

    // Indexing
    FXHVec& __getitem__(FXint i){
      return (*self)[i];
      }

    // Determinant
    FXfloat det() const {
      return det(*self);
      }

    // Transpose
    FXHMat transpose() const {
      return transpose(*self);
      }

    // Invert
    FXHMat invert() const {
      return invert(*self);
      }
    }

  // Set identity matrix
  FXHMat& eye();

  // Orthographic projection
  FXHMat& ortho(FXfloat left,FXfloat right,FXfloat bottom,FXfloat top,FXfloat hither,FXfloat yon);
  
  // Perspective projection
  FXHMat& frustum(FXfloat left,FXfloat right,FXfloat bottom,FXfloat top,FXfloat hither,FXfloat yon);

  // Multiply by left-hand matrix
  FXHMat& left();

  // Multiply by rotation about unit-quaternion
  FXHMat& rot(const FXQuat& q);

  // Multiply by rotation c,s about axis
  // FXHMat& rot(const FXVec& v,FXfloat c,FXfloat s); OVERLOAD

  // Multiply by rotation of phi about axis
  // FXHMat& rot(const FXVec& v,FXfloat phi); OVERLOAD

  // Multiply by x-rotation
  FXHMat& xrot(FXfloat c,FXfloat s);
  // FXHMat& xrot(FXfloat phi); OVERLOAD

  // Multiply by y-rotation
  FXHMat& yrot(FXfloat c,FXfloat s);
  // FXHMat& yrot(FXfloat phi); OVERLOAD

  /// Multiply by z-rotation
  FXHMat& zrot(FXfloat c,FXfloat s);
  // FXHMat& zrot(FXfloat phi);

  // Look at
  FXHMat& look(const FXVec& eye,const FXVec& cntr,const FXVec& vup);

  // Multiply by translation
  FXHMat& trans(FXfloat tx,FXfloat ty,FXfloat tz);
  // FXHMat& trans(const FXVec& v); OVERLOAD

  // Multiply by scaling
  FXHMat& scale(FXfloat sx,FXfloat sy,FXfloat sz);
  // FXHMat& scale(FXfloat s); OVERLOAD
  // FXHMat& scale(const FXVec& v); OVERLOAD

  /// Determinant
  // friend FXfloat det(const FXHMat& m); FRIENDFUNC

  /// Transpose
  // friend FXHMat transpose(const FXHMat& m); FRIENDFUNC

  /// Invert
  // friend FXHMat invert(const FXHMat& m); FRIENDFUNC
  };
