/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/

/// Icon class 
class FXIcon : public FXImage {
protected:
  FXID     shape;             // Shape pixmap
  FXID     etch;              // Etch pixmap
  FXColor  transp;            // Transparency color
protected:
  FXIcon(){}
  FXColor guesstransp(); 
public:

  /**
  * Create an icon with an initial pixel buffer pix, a transparent color clr,
  * and options as in FXImage.
  */
  FXIcon(FXApp* a,const void *pix=NULL,FXColor clr=0,FXuint opts=0,FXint w=1,FXint h=1);

  /// Create the icon resource
  virtual void create();
  
  /// Detach the icon resource
  virtual void detach();
  
  /// Destroy the icon resource
  virtual void destroy();

  /// Render the image from client-side pixel buffer
  virtual void render();

  /// Resize pixmap to the specified width and height
  virtual void resize(FXint w,FXint h);

  /// Destructor
  virtual ~FXIcon();

public:
  %pragma(ruby) markfunc = "FXRbIcon::markfunc";
  };


class FXRbIcon : public FXIcon {
public:
  // Constructor
  FXRbIcon(FXApp* a,const void *pix=NULL,FXColor clr=0,FXuint opts=0,FXint w=1,FXint h=1);

  // Destructor
  virtual ~FXRbIcon();

public:
  DECLARE_FXOBJECT_VIRTUALS
  DECLARE_FXID_VIRTUALS
  DECLARE_FXDRAWABLE_VIRTUALS
  DECLARE_FXIMAGE_VIRTUALS

public:
  %pragma(ruby) markfunc = "FXRbIcon::markfunc";
  };

