/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/

/// JPG Image class
class FXJPGImage : public FXImage {
protected:
  FXint quality;
protected:
  FXJPGImage(){}      
public:
  
  /// Construct an image from memory stream formatted in JPG format
  FXJPGImage(FXApp *a,const void *pix=NULL,FXuint opts=0,FXint w=1,FXint h=1);

  /// Set image quality to save with
  void setQuality(FXint q);

  /// Get image quality setting
  FXint getQuality() const;
  
  /// Destroy
  virtual ~FXJPGImage();

public:
  %pragma(ruby) markfunc = "FXRbJPGImage::markfunc";
  };


class FXRbJPGImage : public FXJPGImage {
public:
  // Constructor
  FXRbJPGImage(FXApp* a,const void *pix=NULL,FXuint opts=0,FXint w=1,FXint h=1);

  // Destructor
  virtual ~FXRbJPGImage();

public:
  DECLARE_FXOBJECT_VIRTUALS
  DECLARE_FXID_VIRTUALS
  DECLARE_FXDRAWABLE_VIRTUALS
  DECLARE_FXIMAGE_VIRTUALS

public:
  %pragma(ruby) markfunc = "FXRbJPGImage::markfunc";
  };


%inline %{
  /// Load a jpeg from a stream
  VALUE fxloadJPG(FXStream& store){
    FXuchar* data;
    FXColor transp;
    FXint width,height,quality;
    if(fxloadJPG(store,data,transp,width,height,quality)){
      VALUE ary=rb_ary_new();
      rb_ary_push(ary,rb_str_new((const char*)data,3*width*height));
      FXFREE(&data);
      rb_ary_push(ary,to_ruby(transp));
      rb_ary_push(ary,to_ruby(width));
      rb_ary_push(ary,to_ruby(height));
      rb_ary_push(ary,to_ruby(quality));
      return ary;
      }
    else{
      return Qnil;
      }
    }

  /// Save a jpeg to a stream
  FXbool fxsaveJPG(FXStream& store,VALUE data,FXColor transp,FXint width,FXint height,FXint quality){
    Check_Type(data,T_STRING);
    const FXuchar* bytes=(const FXuchar*) RSTRING(data)->ptr;
    return fxsaveJPG(store,bytes,transp,width,height,quality);
    }
%}
