/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/

/// States the menu command can be in
enum {
  MENUSTATE_NORMAL   = 0,           /// Normal, unchecked state
  MENUSTATE_CHECKED  = 1,           /// Checked with a checkmark
  MENUSTATE_RCHECKED = 2            /// Checked with a bullet
  };


/**
* The menu command widget is used to invoke a command in the
* application from a menu.  Menu commands may reflect
* the state of the application by graying out, becoming hidden,
* or by a check mark or bullit.
*/
class FXMenuCommand : public FXMenuCaption {
protected:
  FXuchar      state;       // State of menu
  FXString     accel;       // Accelerator string
  FXHotKey     acckey;      // Accelerator key
protected:
  FXMenuCommand(){}
  void drawCheck(FXDCWindow& dc,FXint x,FXint y);
  void drawBullit(FXDCWindow& dc,FXint x,FXint y);
public:
  long onPaint(FXObject*,FXSelector,void* PTR_EVENT);
  long onEnter(FXObject*,FXSelector,void* PTR_EVENT);
  long onLeave(FXObject*,FXSelector,void* PTR_EVENT);
  long onButtonPress(FXObject*,FXSelector,void* PTR_EVENT);
  long onButtonRelease(FXObject*,FXSelector,void* PTR_EVENT);
  long onKeyPress(FXObject*,FXSelector,void* PTR_EVENT);
  long onKeyRelease(FXObject*,FXSelector,void* PTR_EVENT);
  long onHotKeyPress(FXObject*,FXSelector,void* PTR_EVENT);
  long onHotKeyRelease(FXObject*,FXSelector,void* PTR_EVENT);
  long onCheck(FXObject*,FXSelector,void* PTR_IGNORE);
  long onUncheck(FXObject*,FXSelector,void* PTR_IGNORE);
  long onCmdSetValue(FXObject*,FXSelector,void* PTR_BOOL);
  long onCmdSetIntValue(FXObject*,FXSelector,void* PTR_PINT);
  long onCmdGetIntValue(FXObject*,FXSelector,void* PTR_IGNORE); // FIXME
  long onCmdAccel(FXObject*,FXSelector,void* PTR_IGNORE);
public:
  /// Construct a menu command
  FXMenuCommand(FXComposite* p,const FXString& text,FXIcon* ic=NULL,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=0);

  /// Place checkmark next to text
  void check();
  
  /// Uncheck the item
  void uncheck();
  
  /// Return TRUE if checked
  FXint isChecked() const;
  
  /// Place radio bullit next to text
  void checkRadio();
  
  /// Uncheck radio bullit
  void uncheckRadio();
  
  /// Return TRUE if radio-checked
  FXint isRadioChecked() const;
  
  /// Set accelerator text
  void setAccelText(const FXString& text);
  
  /// Return accelarator text
  FXString getAccelText() const;

  /// Destructor
  virtual ~FXMenuCommand();

public:
  %pragma(ruby) markfunc = "FXRbMenuCommand::markfunc";
  };


class FXRbMenuCommand : public FXMenuCommand {
public:
  // Constructor
  FXRbMenuCommand(FXComposite* p,const FXString& text,FXIcon* ic=NULL,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=0);

  // Destructor
  virtual ~FXRbMenuCommand();

public:
  DECLARE_FXOBJECT_VIRTUALS
  DECLARE_FXID_VIRTUALS
  DECLARE_FXDRAWABLE_VIRTUALS
  DECLARE_FXWINDOW_VIRTUALS

public:
  %pragma(ruby) markfunc = "FXRbMenuCommand::markfunc";
  };

