/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/

/// Portable Network Graphics (PNG) Icon
class FXPNGIcon : public FXIcon {
protected:
  FXPNGIcon(){}
public:
  
  /// Construct icon from memory stream formatted in PNG format
  // FXPNGIcon(FXApp* a,const void *pix=NULL,FXColor clr=FXRGB(192,192,192),FXuint opts=0,FXint w=1,FXint h=1); FIXME
  FXPNGIcon(FXApp* a,const void *pix=NULL,FXColor clr=0,FXuint opts=0,FXint w=1,FXint h=1);

  /// Save pixels into stream in PNG format
  virtual void savePixels(FXStream& store) const;
  
  /// Load pixels from stream in PNG format
  virtual void loadPixels(FXStream& store);
  
  /// Destroy icon
  virtual ~FXPNGIcon();

public:
  %pragma(ruby) markfunc = "FXRbPNGIcon::markfunc";
  };


class FXRbPNGIcon : public FXPNGIcon {
public:
  // Constructor
  // FXRbPNGIcon(FXApp* a,const void *pix=NULL,FXColor clr=FXRGB(192,192,192),FXuint opts=0,FXint w=1,FXint h=1); FIXME
  FXRbPNGIcon(FXApp* a,const void *pix=NULL,FXColor clr=0,FXuint opts=0,FXint w=1,FXint h=1);

  // Destructor
  virtual ~FXRbPNGIcon();

public:
  DECLARE_FXOBJECT_VIRTUALS
  DECLARE_FXID_VIRTUALS
  DECLARE_FXDRAWABLE_VIRTUALS
  DECLARE_FXIMAGE_VIRTUALS

public:
  %pragma(ruby) markfunc = "FXRbPNGIcon::markfunc";
  };
  
  
%inline %{
  /// Load a png file from a stream
  VALUE fxloadPNG(FXStream& store){
    FXuchar* data;
    FXColor transp;
    FXint width;
    FXint height;
    if(fxloadPNG(store,data,transp,width,height)){
      VALUE ary=rb_ary_new();
      rb_ary_push(ary,rb_str_new((const char*)data,3*width*height));
      FXFREE(&data);
      rb_ary_push(ary,to_ruby(transp));
      rb_ary_push(ary,to_ruby(width));
      rb_ary_push(ary,to_ruby(height));
      return ary;
      }
    else{
      return Qnil;
      }
    }
%}


/// Save a png file to a stream
FXbool fxsavePNG(FXStream& store,const FXuchar *PIXELS,FXColor transp,FXint width,FXint height);
