/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/

/// Region
class FXRegion {
public:
  
  /// Construct new empty region
  // FXRegion(); OVERLOAD

  /// Construct new region copied from region r
  // FXRegion(const FXRegion& r); OVERLOAD
  
  /// Construct new region set to given rectangle
  FXRegion(FXint x,FXint y,FXint w,FXint h);
  
  /// Assign region r to this one
  // FXRegion &operator=(const FXRegion& r);

  /// Return TRUE if region is empty
  FXbool empty() const;

  /// Return TRUE if region contains point
  FXbool contains(FXint x,FXint y) const;

  /// Return TRUE if region contains rectangle
  // FXbool contains(FXint x,FXint y,FXint w,FXint h) const; FIXME: OVERLOADED

  %addmethods {
    // Return bounding box
    FXRectangle bounds() const {
      FXRectangle r;
      self->bounds(r);
      return r;
    }
  }

  /// Offset region by dx,dy
  FXRegion& offset(FXint dx,FXint dy);

  %addmethods {
    /// Union region r with this one
    FXRegion __add__(const FXRegion& other) {
      return (*self) + other;
    }
  
    /// Intersect region r with this one
    FXRegion __mul__(const FXRegion& other) {
      return (*self) * other;
    }
  
    /// Substract region r from this one
    FXRegion __sub__(const FXRegion& other) {
      return (*self) - other;
    }
  
    /// Xor region r with this one
    FXRegion __xor__(const FXRegion& other) {
      return (*self)^other;
    }

    /// Return TRUE if region equal to this one
    FXbool __eq__(const FXRegion& other) {
      return (*self) == other;
    }
  }

#ifdef SWIGPYTHON
  %addmethods {
    FXbool __ne__(const FXRegion& other) {
      return (*self) != other;
    }
  }
#endif

  /// Destroy region
 ~FXRegion();
  };
