/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/

/// Tree List Box styles
enum {
  TREELISTBOX_NORMAL         = 0          /// Normal style
  };

  
class FXButton;
class FXMenuButton;
class FXTreeList;
class FXPopup;


/// Tree List Box
class FXTreeListBox : public FXPacker {
protected:
  FXButton      *field;
  FXMenuButton  *button;
  FXTreeList    *tree;
  FXPopup       *pane;
protected:
  FXTreeListBox(){}
  virtual void layout();
public:
  long onFocusUp(FXObject*,FXSelector,void* PTR_EVENT);
  long onFocusDown(FXObject*,FXSelector,void* PTR_EVENT);
  long onChanged(FXObject*,FXSelector,void* PTR_TREEITEM);
  long onCommand(FXObject*,FXSelector,void* PTR_TREEITEM);
  long onFieldButton(FXObject*,FXSelector,void* PTR_IGNORE);
  long onTreeChanged(FXObject*,FXSelector,void* PTR_TREEITEM);
  long onTreeClicked(FXObject*,FXSelector,void* PTR_TREEITEM);
  long onUpdFmTree(FXObject*,FXSelector,void* PTR_IGNORE);
public:
  enum{
    ID_TREE=FXPacker::ID_LAST,
    ID_FIELD,
    ID_LAST
    };
public:
  
  /// Construct tree list box
  FXTreeListBox(FXComposite *p,FXint nvis,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=FRAME_SUNKEN|FRAME_THICK|TREELISTBOX_NORMAL,FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint pl=DEFAULT_PAD,FXint pr=DEFAULT_PAD,FXint pt=DEFAULT_PAD,FXint pb=DEFAULT_PAD);

  /// Return number of items
  FXint getNumItems() const;
  
  /// Return number of visible items
  FXint getNumVisible() const;
  
  /// Set number of visible items to determine default height
  void setNumVisible(FXint nvis);
  
  /// Return first top-level item
  FXTreeItem* getFirstItem() const;
  
  /// Return last top-level item
  FXTreeItem* getLastItem() const;

  %addmethods {
    /// Add item as first child of parent p
    FXTreeItem* addItemFirst1(FXTreeItem* p,FXTreeItem* item,void* dummy=NULL){
      if(item->isMemberOf(FXMETACLASS(FXRbTreeItem)))
        ((FXRbTreeItem*)item)->owned=TRUE;
      return self->addItemFirst(p,item);
    }
  
    /// Add item as last child after parent p
    FXTreeItem* addItemLast1(FXTreeItem* p,FXTreeItem* item,void* dummy=NULL){
      if(item->isMemberOf(FXMETACLASS(FXRbTreeItem)))
        ((FXRbTreeItem*)item)->owned=TRUE;
      return self->addItemLast(p,item);
    }
  
    /// Add item after other item
    FXTreeItem* addItemAfter1(FXTreeItem* other,FXTreeItem* item,void* dummy=NULL){
      if(item->isMemberOf(FXMETACLASS(FXRbTreeItem)))
        ((FXRbTreeItem*)item)->owned=TRUE;
      return self->addItemAfter(other,item);
    }
  
    /// Add item before other item
    FXTreeItem* addItemBefore1(FXTreeItem* other,FXTreeItem* item,void* dummy=NULL){
      if(item->isMemberOf(FXMETACLASS(FXRbTreeItem)))
        ((FXRbTreeItem*)item)->owned=TRUE;
      return self->addItemBefore(other,item);
    }
  }
  
  /// Add item as first child of parent p
  %name(addItemFirst2) FXTreeItem* addItemFirst(FXTreeItem* p,const FXString& text,FXIcon* oi=NULL,FXIcon* ci=NULL,void* ITEMDATA=NULL);
  
  /// Add item as last child of parent p
  %name(addItemLast2) FXTreeItem* addItemLast(FXTreeItem* p,const FXString& text,FXIcon* oi=NULL,FXIcon* ci=NULL,void* ITEMDATA=NULL);
  
  /// Add item after other item
  %name(addItemAfter2) FXTreeItem* addItemAfter(FXTreeItem* other,const FXString& text,FXIcon* oi=NULL,FXIcon* ci=NULL,void* ITEMDATA=NULL);
  
  /// Add item before other item
  %name(addItemBefore2) FXTreeItem* addItemBefore(FXTreeItem* other,const FXString& text,FXIcon* oi=NULL,FXIcon* ci=NULL,void* ITEMDATA=NULL);
  
  // Placeholder for overloaded versions of addItemFirst
  FXTreeItem* addItemFirst(FXTreeItem* p,const FXString& text,FXIcon* oi=NULL,FXIcon* ci=NULL,void* ITEMDATA=NULL);

  // Placeholder for overloaded versions of addItemLast
  FXTreeItem* addItemLast(FXTreeItem* p,const FXString& text,FXIcon* oi=NULL,FXIcon* ci=NULL,void* ITEMDATA=NULL);

  // Placeholder for overloaded versions of addItemBefore
  FXTreeItem* addItemBefore(FXTreeItem* other,const FXString& text,FXIcon* oi=NULL,FXIcon* ci=NULL,void* ITEMDATA=NULL);

  // Placeholder for overloaded versions of addItemAfter
  FXTreeItem* addItemAfter(FXTreeItem* other,const FXString& text,FXIcon* oi=NULL,FXIcon* ci=NULL,void* ITEMDATA=NULL);

  /// Remove item
  void removeItem(FXTreeItem* item);
  
  /// Remove all items in range [fm...to]
  void removeItems(FXTreeItem* fm,FXTreeItem* to);
  
  /// Remove all items from list
  void clearItems();
  
  /// Return TRUE if item is the current item
  FXbool isItemCurrent(const FXTreeItem* item) const;
  
  /// Return TRUE if item is leaf-item, i.e. has no children
  FXbool isItemLeaf(const FXTreeItem* item) const;
  
  /// Sort child items of item
  void sortChildItems(FXTreeItem* item);
  
  /// Sort the toplevel items with the sort function
  void sortItems();
  
  /// Change current item
  void setCurrentItem(FXTreeItem* item,FXbool notify=FALSE);
  
  /// Return current item
  FXTreeItem* getCurrentItem() const;
  
  /// Change item label
  void setItemText(FXTreeItem* item,const FXString& text);
  
  /// Return item label
  FXString getItemText(const FXTreeItem* item) const;
  
  /// Change item's open icon
  void setItemOpenIcon(FXTreeItem* item,FXIcon* icon);
  
  /// Return item's open icon
  FXIcon* getItemOpenIcon(const FXTreeItem* item) const;
  
  /// Change item's closed icon
  void setItemClosedIcon(FXTreeItem* item,FXIcon* icon);
  
  /// Return item's closed icon
  FXIcon* getItemClosedIcon(const FXTreeItem* item) const;
  
  %addmethods {
    /// Change item's user data
    void setItemData(FXTreeItem* item,VALUE ptr){
      self->setItemData(item,(void*) ptr);
      }
  
    /// Return item's user data
    VALUE getItemData(const FXTreeItem* item) const {
      return self->getItemData(item) ? (VALUE) self->getItemData(item) : Qnil;
      }
  }
  
  /// Return item sort function
  // FXTreeListSortFunc getSortFunc() const; // FIXME
  
  /// Change item sort function
  // void setSortFunc(FXTreeListSortFunc func); // FIXME

  /// Is the pane shown
  FXbool isPaneShown() const;
  
  /// Change font
  void setFont(FXFont* fnt);
  
  /// Return font
  FXFont* getFont() const;
  
  /// Return list style
  FXuint getListStyle() const;
  
  /// Change list style
  void setListStyle(FXuint style);
  
  /// Change help text
  void setHelpText(const FXString& txt);
  
  /// Return help text
  FXString getHelpText() const;
  
  /// Change tip text
  void setTipText(const FXString& txt);
  
  /// Return tip text
  FXString getTipText() const;

  /// Destructor
  virtual ~FXTreeListBox();

public:
  %pragma(ruby) markfunc = "FXRbTreeListBox::markfunc";
  };


class FXRbTreeListBox : public FXTreeListBox {
public:
  // Construct tree list box
  FXRbTreeListBox(FXComposite *p,FXint nvis,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=FRAME_SUNKEN|FRAME_THICK|TREELISTBOX_NORMAL,FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint pl=DEFAULT_PAD,FXint pr=DEFAULT_PAD,FXint pt=DEFAULT_PAD,FXint pb=DEFAULT_PAD);

  // Destructor
  virtual ~FXRbTreeListBox();

public:
  DECLARE_FXOBJECT_VIRTUALS
  DECLARE_FXID_VIRTUALS
  DECLARE_FXDRAWABLE_VIRTUALS
  DECLARE_FXWINDOW_VIRTUALS

public:
  %pragma(ruby) markfunc = "FXRbTreeListBox::markfunc";
  };

