/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/

class FXVec {
public:
  // Default constructor
  FXVec(FXfloat x=0.0f,FXfloat y=0.0f,FXfloat z=0.0f);

  // Other good stuff
  %addmethods {
    FXfloat __getitem__(FXint i){
      if(i<0||i>2){
        rb_raise(rb_eIndexError,"index %d out of range",i);
        return 0.0f;
        }
      else
        return (*self)[i];
      }
    void __setitem__(FXint i,FXfloat f){
      if(i<0||i>2)
        rb_raise(rb_eIndexError,"index %d out of range",i);
      else
        (*self)[i]=f;
      }
    FXbool __eq__(const FXVec& other){
      return (*self == other);
      }
    FXVec __neg__(){
      return -(*self);
      }
    FXVec __add__(const FXVec& other){
      return *self + other;
      }
    FXVec __sub__(const FXVec& other){
      return *self - other;
      }
    FXVec __mul__(FXfloat n){
      return (*self)*n;
      }
    FXVec __div__(FXfloat n){
      return (*self)/n;
      }
    FXfloat dot(const FXVec& other){
      return (*self)*other;
      }
    FXVec cross(const FXVec& other){
      return (*self)^other;
      }
    FXfloat len(){
      return len(*self);
      }
    static FXVec normalize(const FXVec& vec){
      return normalize(vec);
      }
    static FXVec lo(const FXVec& a,const FXVec& b){
      return lo(a,b);
      }
    static FXVec hi(const FXVec& a,const FXVec& b){
      return hi(a,b);
      }
    VALUE to_a(){
      VALUE ary=rb_ary_new();
      rb_ary_push(ary,to_ruby((*self)[0]));
      rb_ary_push(ary,to_ruby((*self)[1]));
      rb_ary_push(ary,to_ruby((*self)[2]));
      return ary;
      }
    }
  };
