/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/

#define DECLARE_FXAPP_VIRTUALS \
    %name(create) void _create(); \
    %name(detach) void _detach(); \
    %name(destroy) void _destroy(); \
    %name(init) void _init(VALUE arr,FXbool connect=TRUE); \
    %name(exit) void _exit(FXint code=0);

#define DECLARE_FXBITMAP_VIRTUALS \
    %name(render) void _render(); \
    %name(savePixels) void _savePixels(FXStream& store) const; \
    %name(loadPixels) void _loadPixels(FXStream& store);

#define DECLARE_FXCOMBOBOX_VIRTUALS \
    %name(setBackColor) void _setBackColor(FXColor clr);

#define DECLARE_FXCURSOR_VIRTUALS \
    %name(savePixels) void _savePixels(FXStream& store) const; \
    %name(loadPixels) void _loadPixels(FXStream& store);

#define DECLARE_FXDC_VIRTUALS \
    %name(readPixel) FXColor _readPixel(FXint x,FXint y); \
    %name(drawPoint) void _drawPoint(FXint x,FXint y); \
    %name(drawPoints) void _drawPoints(const FXPoint* points,FXuint npoints); \
    %name(drawPointsRel) void _drawPointsRel(const FXPoint* points,FXuint npoints); \
    %name(drawLine) void _drawLine(FXint x1,FXint y1,FXint x2,FXint y2); \
    %name(drawLines) void _drawLines(const FXPoint* points,FXuint npoints); \
    %name(drawLinesRel) void _drawLinesRel(const FXPoint* points,FXuint npoints); \
    %name(drawLineSegments) void _drawLineSegments(const FXSegment* segments,FXuint nsegments); \
    %name(drawRectangle) void _drawRectangle(FXint x,FXint y,FXint w,FXint h); \
    %name(drawRectangles) void _drawRectangles(const FXRectangle* rectangles,FXuint nrectangles); \
    %name(drawArc) void _drawArc(FXint x,FXint y,FXint w,FXint h,FXint ang1,FXint ang2); \
    %name(drawArcs) void _drawArcs(const FXArc* arcs,FXuint narcs); \
    %name(fillRectangle) void _fillRectangle(FXint x,FXint y,FXint w,FXint h); \
    %name(fillRectangles) void _fillRectangles(const FXRectangle* rectangles,FXuint nrectangles); \
    %name(fillArc) void _fillArc(FXint x,FXint y,FXint w,FXint h,FXint ang1,FXint ang2); \
    %name(fillArcs) void _fillArcs(const FXArc* arcs,FXuint narcs); \
    %name(fillPolygon) void _fillPolygon(const FXPoint* points,FXuint npoints); \
    %name(fillConcavePolygon) void _fillConcavePolygon(const FXPoint* points,FXuint npoints); \
    %name(fillComplexPolygon) void _fillComplexPolygon(const FXPoint* points,FXuint npoints); \
    %name(fillPolygonRel) void _fillPolygonRel(const FXPoint* points,FXuint npoints); \
    %name(fillConcavePolygonRel) void _fillConcavePolygonRel(const FXPoint* points,FXuint npoints); \
    %name(fillComplexPolygonRel) void _fillComplexPolygonRel(const FXPoint* points,FXuint npoints); \
    %name(drawHashBox) void _drawHashBox(FXint x,FXint y,FXint w,FXint h,FXint b=1); \
    %name(drawFocusRectangle) void _drawFocusRectangle(FXint x,FXint y,FXint w,FXint h); \
    %name(drawArea) void _drawArea(const FXDrawable* source,FXint sx,FXint sy,FXint sw,FXint sh,FXint dx,FXint dy); \
    %name(drawImage) void _drawImage(const FXImage* image,FXint dx,FXint dy); \
    %name(drawBitmap) void _drawBitmap(const FXBitmap* bitmap,FXint dx,FXint dy); \
    %name(drawIcon) void _drawIcon(const FXIcon* icon,FXint dx,FXint dy); \
    %name(drawIconShaded) void _drawIconShaded(const FXIcon* icon,FXint dx,FXint dy); \
    %name(drawIconSunken) void _drawIconSunken(const FXIcon* icon,FXint dx,FXint dy); \
    %name(drawText) void _drawText(FXint x,FXint y,const FXchar* string,FXuint length); \
    %name(drawImageText) void _drawImageText(FXint x,FXint y,const FXchar* string,FXuint length); \
    %name(setForeground) void _setForeground(FXColor clr); \
    %name(setBackground) void _setBackground(FXColor clr); \
    %name(setDashes) void _setDashes(FXuint dashoffset,const FXchar *dashpattern,FXuint dashlength); \
    %name(setLineWidth) void _setLineWidth(FXuint linewidth=0); \
    %name(setLineCap) void _setLineCap(FXCapStyle capstyle=CAP_BUTT); \
    %name(setLineJoin) void _setLineJoin(FXJoinStyle joinstyle=JOIN_MITER); \
    %name(setLineStyle) void _setLineStyle(FXLineStyle linestyle=LINE_SOLID); \
    %name(setFillStyle) void _setFillStyle(FXFillStyle fillstyle=FILL_SOLID); \
    %name(setFillRule) void _setFillRule(FXFillRule fillrule=RULE_EVEN_ODD); \
    %name(setFunction) void _setFunction(FXFunction func=BLT_SRC); \
    %name(setTile) void _setTile(FXImage* image,FXint dx=0,FXint dy=0); \
    %name(setStipple) void _setStipple(FXStipplePattern stipple,FXint dx=0,FXint dy=0); \
    %name(setClipRegion) void _setClipRegion(const FXRegion& region); \
    %name(setClipRectangle) void _setClipRectangle(FXint x,FXint y,FXint w,FXint h); \
    %name(setClipRectangle2) void _setClipRectangle(const FXRectangle& rectangle); \
    %name(clearClipRectangle) void _clearClipRectangle(); \
    %name(setClipMask) void _setClipMask(FXBitmap* bitmap,FXint dx=0,FXint dy=0); \
    %name(clearClipMask) void _clearClipMask(); \
    %name(setTextFont) void _setTextFont(FXFont *fnt); \
    %name(clipChildren) void _clipChildren(FXbool yes);

#define DECLARE_FXDIALOGBOX_VIRTUALS \
    %name(execute) FXuint _execute(FXuint placement=PLACEMENT_CURSOR);

#define DECLARE_FXDRAWABLE_VIRTUALS \
	%name(resize) void _resize(FXint w,FXint h);

#define DECLARE_FXFILEDICT_VIRTUALS \
    %name(findFileBinding) FXFileAssoc* _findFileBinding(const FXchar* pathname); \
    %name(findDirBinding)  FXFileAssoc* _findDirBinding(const FXchar* pathname); \
    %name(findExecBinding) FXFileAssoc* _findExecBinding(const FXchar* pathname);

#define DECLARE_FXGLCANVAS_VIRTUALS \
    %name(makeCurrent) FXbool _makeCurrent(); \
    %name(makeNonCurrent) FXbool _makeNonCurrent(); \
    %name(isCurrent) FXbool _isCurrent() const; \
    %name(swapBuffers) void _swapBuffers();

#define DECLARE_FXGLOBJECT_VIRTUALS \
    %name(bounds) FXRange _bounds(); \
    %name(copy) void _copy(); \
    %name(draw) void _draw(FXGLViewer* viewer); \
    %name(hit) void _hit(FXGLViewer* viewer); \
    %name(identify) FXGLObject* _identify(FXuint* path); \
    %name(canDrag) FXbool _canDrag() const; \
    %name(canDelete) FXbool _canDelete() const; \
    %name(drag) FXbool _drag(FXGLViewer* viewer,FXint fx,FXint fy,FXint tx,FXint ty);

#define DECLARE_FXGLVIEWER_VIRTUALS \
    %name(pick) FXGLObject* _pick(FXint x,FXint y);
    // %name(select) FXGLObject** _select(FXint x,FXint y,FXint w,FXint h); \

#define DECLARE_FXHEADERITEM_VIRTUALS \
    %name(setText) void _setText(const FXString& txt); \
    %name(setIcon) void _setIcon(FXIcon *ic); \
    %name(getWidth) FXint getWidth(const FXHeader* header) const; \
    %name(getHeight) FXint getHeight(const FXHeader* header) const; \
    %name(create) void _create(); \
    %name(detach) void _detach(); \
    %name(destroy) void _destroy();

#define DECLARE_FXICONITEM_VIRTUALS \
    %name(setBigIcon) void _setBigIcon(FXIcon* icn); \
    %name(setMiniIcon) void _setMiniIcon(FXIcon* icn); \
    %name(setFocus) void _setFocus(FXbool focus); \
    %name(setSelected) void _setSelected(FXbool selected); \
    %name(setEnabled) void _setEnabled(FXbool enabled); \
    %name(setDraggable) void _setDraggable(FXbool draggable); \
    %name(setIconOwned) void _setIconOwned(FXuint owned); \
    %name(getWidth) FXint getWidth(const FXIconList* list) const; \
    %name(getHeight) FXint getHeight(const FXIconList* list) const; \
    %name(create) void _create(); \
    %name(detach) void _detach(); \
    %name(destroy) void _destroy();

#define DECLARE_FXID_VIRTUALS \
    %name(create) void _create(); \
    %name(detach) void _detach(); \
    %name(destroy) void _destroy();

#define DECLARE_FXIMAGE_VIRTUALS \
    %name(restore) void _restore(); \
    %name(render) void _render(); \
    %name(scale) void _scale(FXint w,FXint h); \
    %name(mirror) void _mirror(FXbool horizontal,FXbool vertical); \
    %name(rotate) void _rotate(FXint degrees); \
    %name(crop) void _crop(FXint x,FXint y,FXint w,FXint h); \
    %name(savePixels) void _savePixels(FXStream& store) const; \
    %name(loadPixels) void _loadPixels(FXStream& store);

#define DECLARE_FXLISTBOX_VIRTUALS \
    %name(setBackColor) void _setBackColor(FXColor clr);

#define DECLARE_FXLISTITEM_VIRTUALS \
    %name(setText) void _setText(const FXString& txt); \
    %name(setIcon) void _setIcon(FXIcon* icn); \
    %name(setFocus) void _setFocus(FXbool focus); \
    %name(setSelected) void _setSelected(FXbool selected); \
    %name(setEnabled) void _setEnabled(FXbool enabled); \
    %name(setDraggable) void _setDraggable(FXbool draggable); \
    %name(setIconOwned) void _setIconOwned(FXuint owned); \
    %name(getWidth) FXint _getWidth(const FXList* list) const; \
    %name(getHeight) FXint _getHeight(const FXList* list) const; \
    %name(create) void _create(); \
    %name(detach) void _detach(); \
    %name(destroy) void _destroy();

#define DECLARE_FXMDICHILD_VIRTUALS \
    %name(minimize) void _minimize(FXbool notify=FALSE); \
    %name(maximize) void _maximize(FXbool notify=FALSE); \
    %name(restore) void _restore(FXbool notify=FALSE);

#define DECLARE_FXOBJECT_VIRTUALS \
    %name(onDefault) long _onDefault(FXObject* sender,FXSelector sel,void* ptr); \
    %name(save) void _save(FXStream& store) const; \
    %name(load) void _load(FXStream& store);

#define DECLARE_FXPOPUP_VIRTUALS \
    %name(popup) void _popup(FXWindow* grabto,FXint x,FXint y,FXint w=0,FXint h=0); \
    %name(popdown) void _popdown();

#define DECLARE_FXSCROLLAREA_VIRTUALS \
    %name(getViewportHeight) FXint _getViewportHeight(); \
    %name(getViewportWidth) FXint _getViewportWidth(); \
    %name(getContentHeight) FXint _getContentHeight(); \
    %name(getContentWidth) FXint _getContentWidth(); \
    %name(moveContents) void _moveContents(FXint x,FXint y);

#define DECLARE_FXSPINNER_VIRTUALS \
    %name(setValue) void _setValue(FXint value);

#define DECLARE_FXSHUTTER_VIRTUALS \
    %name(setCurrent) void _setCurrent(FXint panel);

#define DECLARE_FXTABBAR_VIRTUALS \
    %name(setCurrent) void _setCurrent(FXint panel,FXbool notify=FALSE);

#define DECLARE_FXTABLEITEM_VIRTUALS \
    %name(setText) void _setText(const FXString& txt); \
    %name(setIcon) void _setIcon(FXIcon* icn); \
    %name(setFocus) void _setFocus(FXbool focus); \
    %name(setSelected) void _setSelected(FXbool selected); \
    %name(setEnabled) void _setEnabled(FXbool enabled); \
    %name(setDraggable) void _setDraggable(FXbool draggable); \
    %name(setIconOwned) void _setIconOwned(FXuint owned); \
    %name(getWidth) FXint _getWidth(const FXTable* table) const; \
    %name(getHeight) FXint _getHeight(const FXTable* table) const; \
    %name(create) void _create(); \
    %name(detach) void _detach(); \
    %name(destroy) void _destroy();

#define DECLARE_FXTABLE_VIRTUALS \
    %name(setLeadingRows) void _setLeadingRows(FXint leadrows); \
    %name(setLeadingCols) void _setLeadingCols(FXint leadcols); \
    %name(setTrailingRows) void _setTrailingRows(FXint trailrows); \
    %name(setTrailingCols) void _setTrailingCols(FXint trailcols); \
    %name(insertRows) void _insertRows(FXint row,FXint nr=1,FXbool notify=FALSE); \
    %name(insertColumns) void _insertColumns(FXint col,FXint nc=1,FXbool notify=FALSE); \
    %name(removeRows) void _removeRows(FXint row,FXint nr=1,FXbool notify=FALSE); \
    %name(removeColumns) void _removeColumns(FXint col,FXint nc=1,FXbool notify=FALSE); \
    %name(removeItem) void _removeItem(FXint row,FXint col,FXbool notify=FALSE);

#define DECLARE_FXTEXT_VIRTUALS \
    %name(setCursorPos) void _setCursorPos(FXint pos,FXbool notify=FALSE); \
    %name(extendSelection) FXbool _extendSelection(FXint pos,FXTextSelectionMode mode=SELECT_CHARS,FXbool notify=FALSE); \
    %name(killSelection) FXbool _killSelection(FXbool notify=FALSE);

#define DECLARE_FXTOOLBAR_VIRTUALS \
    %name(dock) void _dock(FXuint side,FXWindow* after); \
    %name(undock) void _undock();

/* Don't declare the stubbed out show() with placement (special case) */
#define DECLARE_FXTOPWINDOW_VIRTUALS \
    %name(iconify) void _iconify(); \
    %name(deiconify) void _deiconify();

#define DECLARE_FXTREEITEM_VIRTUALS \
    %name(setText) void _setText(const FXString& txt); \
    %name(setOpenIcon) void _setOpenIcon(FXIcon* icn); \
    %name(setClosedIcon) void _setClosedIcon(FXIcon* icn); \
    %name(setFocus) void _setFocus(FXbool focus); \
    %name(setSelected) void _setSelected(FXbool selected); \
    %name(setOpened) void _setOpened(FXbool opened); \
    %name(setExpanded) void _setExpanded(FXbool expanded); \
    %name(setEnabled) void _setEnabled(FXbool enabled); \
    %name(setDraggable) void _setDraggable(FXbool draggable); \
    %name(setIconOwned) void _setIconOwned(FXuint owned); \
    %name(getWidth) FXint _getWidth(const FXTreeList* list) const; \
    %name(getHeight) FXint _getHeight(const FXTreeList* list) const; \
    %name(create) void _create(); \
    %name(detach) void _detach(); \
    %name(destroy) void _destroy();

#define DECLARE_FXTREELIST_VIRTUALS \
    %name(getItemAt) FXTreeItem* getItemAt(FXint x,FXint y) const;

#define DECLARE_FXWINDOW_VIRTUALS \
    %name(getDefaultWidth) FXint _getDefaultWidth(); \
    %name(getDefaultHeight) FXint _getDefaultHeight(); \
    %name(getWidthForHeight) FXint _getWidthForHeight(FXint givenheight); \
    %name(getHeightForWidth) FXint _getHeightForWidth(FXint givenwidth); \
    %name(canFocus) FXbool _canFocus() const; \
    %name(setFocus) void _setFocus(); \
    %name(killFocus) void _killFocus(); \
    %name(setDefault) void _setDefault(FXbool enable=TRUE); \
    %name(enable) void _enable(); \
    %name(disable) void _disable(); \
    %name(raise) void _raise(); \
    %name(lower) void _lower(); \
    %name(move) void _move(FXint x,FXint y); \
    %name(position) void _position(FXint x,FXint y,FXint w,FXint h); \
    %name(recalc) void _recalc(); \
    %name(reparent) void _reparent(FXComposite* newparent); \
    %name(show) void _show(); \
    %name(hide) void _hide(); \
    %name(isComposite) FXbool _isComposite() const; \
    %name(contains) FXbool _contains(FXint parentx,FXint parenty) const; \
    %name(doesSaveUnder) FXbool _doesSaveUnder() const; \
    %name(setBackColor) void _setBackColor(FXColor clr); \
    %name(layout) void _layout();

#define DECLARE_FXGLSHAPE_VIRTUALS \
    %name(drawshape) void _drawshape(FXGLViewer* viewer);
